@@ -18,6 +18,7 @@
 #include "AxisVelocityCalculator.h"
 #include "Direction.h"
 #include "DirectionalDragArea.h"
+#include "PressedOutsideNotifier.h"
 
 #include <qqml.h>
 
@@ -32,4 +33,5 @@ void UbuntuGestureQmlPlugin::registerTypes(const char *uri)
     qmlRegisterSingletonType<Direction>(uri, 0, 1, "Direction", directionSingleton);
     qmlRegisterType<DirectionalDragArea>(uri, 0, 1, "DirectionalDragArea");
     qmlRegisterType<AxisVelocityCalculator>(uri, 0, 1, "AxisVelocityCalculator");
+    qmlRegisterType<PressedOutsideNotifier>(uri, 0, 1, "PressedOutsideNotifier");
 }
