@@ -1,7 +1,7 @@
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 #
 # Unity Autopilot Test Suite
-# Copyright (C) 2012-2013 Canonical
+# Copyright (C) 2012, 2013, 2014 Canonical
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,9 +17,12 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-
-from autopilot.introspection import CustomEmulatorBase
+from ubuntuuitoolkit import emulators as toolkit_emulators
 
 
-class UnityEmulatorBase(CustomEmulatorBase):
+class UnityEmulatorBase(toolkit_emulators.UbuntuUIToolkitEmulatorBase):
     """A base class for all unity emulators."""
+
+
+class UnityEmulatorException(Exception):
+    """Exception raised when there is an error with the Unity emulators."""
