@@ -1,7 +1,7 @@
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 #
 # Unity Autopilot Test Suite
-# Copyright (C) 2012-2013 Canonical
+# Copyright (C) 2012, 2013, 2014 Canonical
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,24 +18,21 @@
 #
 
 from unity8.shell.emulators import UnityEmulatorBase
-from autopilot.input import Touch
 
 
 class Greeter(UnityEmulatorBase):
-
     """An emulator that understands the greeter screen."""
 
     def swipe(self):
         """Swipe the greeter screen away."""
         self.created.wait_for(True)
-        touch = Touch.create()
 
         rect = self.globalRect
         start_x = rect[0] + rect[2] - 3
         start_y = int(rect[1] + rect[3] / 2)
         stop_x = int(rect[0] + rect[2] * 0.2)
         stop_y = start_y
-        touch.drag(start_x, start_y, stop_x, stop_y)
+        self.pointing_device.drag(start_x, start_y, stop_x, stop_y)
 
         self.created.wait_for(False)
 
