@@ -1,7 +1,7 @@
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 #
 # Unity Autopilot Test Suite
-# Copyright (C) 2012-2013 Canonical
+# Copyright (C) 2012, 2013, 2014 Canonical
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 
 from __future__ import absolute_import
 
+from unity8.process_helpers import unlock_unity
 from unity8.shell.tests import UnityTestCase, _get_device_emulation_scenarios
 
 from testtools.matchers import Equals, NotEquals
@@ -32,9 +33,15 @@ import os
 import logging
 import signal
 import subprocess
+import sys
 
 logger = logging.getLogger(__name__)
 
+# from __future__ import range
+# (python3's range, is same as python2's xrange)
+if sys.version_info < (3,):
+    range = xrange
+
 
 class NotificationsBase(UnityTestCase):
     """Base class for all notification tests that provides helper methods."""
@@ -45,7 +52,7 @@ class NotificationsBase(UnityTestCase):
         """Given an icons file name returns the full path (either system or
         source tree.
 
-        Consider the graphics directory as root so for example (runnign tests
+        Consider the graphics directory as root so for example (running tests
         from installed unity8-autopilot package):
         >>> self.get_icon_path('clock.png')
         /usr/share/unity8/graphics/clock.png
@@ -57,14 +64,11 @@ class NotificationsBase(UnityTestCase):
         if os.path.abspath(__file__).startswith('/usr/'):
             return '/usr/share/unity8/graphics/' + icon_name
         else:
-            return os.path.abspath(
-                os.getcwd() + "/../../graphics/" + icon_name
-            )
+            return os.path.dirname(__file__) + "/../../../../../qml/graphics/" + icon_name
 
     def _get_notifications_list(self):
-        main_view = self.main_window.get_qml_view()
-        return main_view.select_single(
-            "QQuickListView",
+        return self.main_window.select_single(
+            "Notifications",
             objectName='notificationList'
         )
 
@@ -118,9 +122,8 @@ class InteractiveNotificationBase(NotificationsBase):
 
     def test_interactive(self):
         """Interactive notification must react upon click on itself."""
-        self.launch_unity()
-        greeter = self.main_window.get_greeter()
-        greeter.swipe()
+        unity_proxy = self.launch_unity()
+        unlock_unity(unity_proxy)
 
         notify_list = self._get_notifications_list()
 
@@ -145,8 +148,8 @@ class InteractiveNotificationBase(NotificationsBase):
             hints,
         )
 
-        get_notification = lambda: notify_list.select_single('Notification', objectName='notification1')
-        self.assertThat(get_notification, Eventually(NotEquals(None)))
+        get_notification = lambda: notify_list.wait_select_single(
+            'Notification', objectName='notification1')
         notification = get_notification()
 
         self.touch.tap_object(
@@ -156,12 +159,10 @@ class InteractiveNotificationBase(NotificationsBase):
         self.assert_notification_action_id_was_called('action_id')
 
     def test_sd_incoming_call(self):
-        """Rejecting a call should make notification expand and offer more options."""
-        self.launch_unity()
-        greeter = self.main_window.get_greeter()
-        greeter.swipe()
-
-        notify_list = self._get_notifications_list()
+        """Rejecting a call should make notification expand and
+            offer more options."""
+        unity_proxy = self.launch_unity()
+        unlock_unity(unity_proxy)
 
         summary = "Incoming call"
         body = "Frank Zappa\n+44 (0)7736 027340"
@@ -191,17 +192,20 @@ class InteractiveNotificationBase(NotificationsBase):
             hints
         )
 
-        get_notification = lambda: notify_list.select_single('Notification', objectName='notification1')
-        self.assertThat(get_notification, Eventually(NotEquals(None)))
+        notify_list = self._get_notifications_list()
+        get_notification = lambda: notify_list.wait_select_single(
+            'Notification', objectName='notification1')
         notification = get_notification()
-        self._assert_notification(notification, None, None, True, True, 1.0)
+        self._assert_notification(notification, summary, body, True, True, 1.0)
         initial_height = notification.height
         self.touch.tap_object(notification.select_single(objectName="button1"))
         self.assertThat(
             notification.height,
             Eventually(Equals(initial_height +
-                              3 * notification.select_single(objectName="buttonColumn").spacing +
-                              3 * notification.select_single(objectName="button4").height)))
+                              3 * notification.select_single(
+                                  objectName="buttonColumn").spacing +
+                              3 * notification.select_single(
+                                  objectName="button4").height)))
         self.touch.tap_object(notification.select_single(objectName="button4"))
         self.assert_notification_action_id_was_called("action_decline_4")
 
@@ -263,6 +267,7 @@ class InteractiveNotificationBase(NotificationsBase):
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE,
             close_fds=True,
+            universal_newlines=True,
         )
 
         self.addCleanup(self._tidy_up_script_process)
@@ -300,7 +305,7 @@ class InteractiveNotificationBase(NotificationsBase):
         if self._notify_proc is None:
             raise AssertionError("No interactive notification was created.")
 
-        for i in xrange(timeout):
+        for i in range(timeout):
             self._notify_proc.poll()
             if self._notify_proc.returncode is not None:
                 output = self._notify_proc.communicate()
@@ -333,9 +338,8 @@ class EphemeralNotificationsTests(NotificationsBase):
 
     def test_icon_summary_body(self):
         """Notification must display the expected summary and body text."""
-        self.launch_unity()
-        greeter = self.main_window.get_greeter()
-        greeter.swipe()
+        unity_proxy = self.launch_unity()
+        unlock_unity(unity_proxy)
 
         notify_list = self._get_notifications_list()
 
@@ -360,16 +364,22 @@ class EphemeralNotificationsTests(NotificationsBase):
 
         notification.show()
 
-        get_notification = lambda: notify_list.select_single('Notification', objectName='notification1')
-        self.assertThat(get_notification, Eventually(NotEquals(None)))
-        notification = get_notification()
-        self._assert_notification(notification, summary, body, True, True, 1.0)
+        notification = lambda: notify_list.wait_select_single(
+            'Notification', objectName='notification1')
+        self._assert_notification(
+            notification(),
+            summary,
+            body,
+            True,
+            True,
+            1.0,
+        )
 
     def test_icon_summary(self):
-        """Notification must display the expected summary and secondary icon."""
-        self.launch_unity()
-        greeter = self.main_window.get_greeter()
-        greeter.swipe()
+        """Notification must display the expected summary and secondary
+        icon."""
+        unity_proxy = self.launch_unity()
+        unlock_unity(unity_proxy)
 
         notify_list = self._get_notifications_list()
 
@@ -391,11 +401,10 @@ class EphemeralNotificationsTests(NotificationsBase):
 
         notification.show()
 
-        get_notification = lambda: notify_list.select_single('Notification', objectName='notification1')
-        self.assertThat(get_notification, Eventually(NotEquals(None)))
-        notification = get_notification()
+        notification = lambda: notify_list.wait_select_single(
+            'Notification', objectName='notification1')
         self._assert_notification(
-            notification,
+            notification(),
             summary,
             None,
             False,
@@ -404,10 +413,10 @@ class EphemeralNotificationsTests(NotificationsBase):
         )
 
     def test_urgency_order(self):
-        """Notifications must be displayed in order according to their urgency."""
-        self.launch_unity()
-        greeter = self.main_window.get_greeter()
-        greeter.swipe()
+        """Notifications must be displayed in order according to their
+        urgency."""
+        unity_proxy = self.launch_unity()
+        unlock_unity(unity_proxy)
 
         notify_list = self._get_notifications_list()
 
@@ -448,11 +457,10 @@ class EphemeralNotificationsTests(NotificationsBase):
         )
         notification_critical.show()
 
-        get_notification = lambda: notify_list.select_single(
+        get_notification = lambda: notify_list.wait_select_single(
             'Notification',
             summary=summary_critical
         )
-        self.assertThat(get_notification, Eventually(NotEquals(None)))
 
         notification = get_notification()
         self._assert_notification(
@@ -464,11 +472,10 @@ class EphemeralNotificationsTests(NotificationsBase):
             1.0
         )
 
-        get_normal_notification = lambda: notify_list.select_single(
+        get_normal_notification = lambda: notify_list.wait_select_single(
             'Notification',
             summary=summary_normal
         )
-        self.assertThat(get_normal_notification, Eventually(NotEquals(None)))
         notification = get_normal_notification()
         self._assert_notification(
             notification,
@@ -479,11 +486,10 @@ class EphemeralNotificationsTests(NotificationsBase):
             1.0
         )
 
-        get_low_notification = lambda: notify_list.select_single(
+        get_low_notification = lambda: notify_list.wait_select_single(
             'Notification',
             summary=summary_low
         )
-        self.assertThat(get_low_notification, Eventually(NotEquals(None)))
         notification = get_low_notification()
         self._assert_notification(
             notification,
@@ -496,9 +502,8 @@ class EphemeralNotificationsTests(NotificationsBase):
 
     def test_summary_and_body(self):
         """Notification must display the expected summary- and body-text."""
-        self.launch_unity()
-        greeter = self.main_window.get_greeter()
-        greeter.swipe()
+        unity_proxy = self.launch_unity()
+        unlock_unity(unity_proxy)
 
         notify_list = self._get_notifications_list()
 
@@ -508,9 +513,9 @@ class EphemeralNotificationsTests(NotificationsBase):
         notification = self._create_ephemeral_notification(summary, body)
         notification.show()
 
-        get_notification = lambda: notify_list.select_single('Notification', objectName='notification1')
-        self.assertThat(get_notification, Eventually(NotEquals(None)))
-        notification = get_notification()
+        notification = notify_list.wait_select_single(
+            'Notification', objectName='notification1')
+
         self._assert_notification(
             notification,
             summary,
@@ -522,9 +527,8 @@ class EphemeralNotificationsTests(NotificationsBase):
 
     def test_summary_only(self):
         """Notification must display only the expected summary-text."""
-        self.launch_unity()
-        greeter = self.main_window.get_greeter()
-        greeter.swipe()
+        unity_proxy = self.launch_unity()
+        unlock_unity(unity_proxy)
 
         notify_list = self._get_notifications_list()
 
@@ -533,17 +537,16 @@ class EphemeralNotificationsTests(NotificationsBase):
         notification = self._create_ephemeral_notification(summary)
         notification.show()
 
-        get_notification = lambda: notify_list.select_single('Notification', objectName='notification1')
-        self.assertThat(get_notification, Eventually(NotEquals(None)))
-        notification = get_notification()
+        notification = notify_list.wait_select_single(
+            'Notification', objectName='notification1')
 
         self._assert_notification(notification, summary, '', False, False, 1.0)
 
     def test_update_notification_same_layout(self):
-        """Notification must allow updating its contents while being displayed."""
-        self.launch_unity()
-        greeter = self.main_window.get_greeter()
-        greeter.swipe()
+        """Notification must allow updating its contents while being
+        displayed."""
+        unity_proxy = self.launch_unity()
+        unlock_unity(unity_proxy)
 
         notify_list = self._get_notifications_list()
 
@@ -558,8 +561,8 @@ class EphemeralNotificationsTests(NotificationsBase):
         )
         notification.show()
 
-        get_notification = lambda: notify_list.select_single('Notification', summary=summary)
-        self.assertThat(get_notification, Eventually(NotEquals(None)))
+        get_notification = lambda: notify_list.wait_select_single(
+            'Notification', summary=summary)
         self._assert_notification(
             get_notification(),
             summary,
@@ -575,14 +578,13 @@ class EphemeralNotificationsTests(NotificationsBase):
         icon_path = self._get_icon_path('avatars/amanda.png')
         notification.update(summary, body, icon_path)
         notification.show()
-        self.assertThat(get_notification, Eventually(NotEquals(None)))
-        self._assert_notification(get_notification(), summary, body)
+        self._assert_notification(get_notification(), summary, body, True, False, 1.0)
 
     def test_update_notification_layout_change(self):
-        """Notification must allow updating its contents and layout while being displayed."""
-        self.launch_unity()
-        greeter = self.main_window.get_greeter()
-        greeter.swipe()
+        """Notification must allow updating its contents and layout while
+        being displayed."""
+        unity_proxy = self.launch_unity()
+        unlock_unity(unity_proxy)
 
         notify_list = self._get_notifications_list()
 
@@ -603,8 +605,9 @@ class EphemeralNotificationsTests(NotificationsBase):
         )
         notification.show()
 
-        get_notification = lambda: notify_list.select_single('Notification', objectName='notification1')
-        self.assertThat(get_notification, Eventually(NotEquals(None)))
+        get_notification = lambda: notify_list.wait_select_single(
+            'Notification', objectName='notification1')
+
         self._assert_notification(
             get_notification(),
             summary,
@@ -622,13 +625,12 @@ class EphemeralNotificationsTests(NotificationsBase):
         notification.show()
 
         self.assertThat(get_notification, Eventually(NotEquals(None)))
-        self._assert_notification(get_notification(), summary, body, False)
+        self._assert_notification(get_notification(), summary, body, False, False, 1.0)
 
     def test_append_hint(self):
         """Notification has to accumulate body-text using append-hint."""
-        self.launch_unity()
-        greeter = self.main_window.get_greeter()
-        greeter.swipe()
+        unity_proxy = self.launch_unity()
+        unlock_unity(unity_proxy)
 
         notify_list = self._get_notifications_list()
 
@@ -645,8 +647,9 @@ class EphemeralNotificationsTests(NotificationsBase):
 
         notification.show()
 
-        get_notification = lambda: notify_list.select_single('Notification', objectName='notification1')
-        self.assertThat(get_notification, Eventually(NotEquals(None)))
+        get_notification = lambda: notify_list.wait_select_single(
+            'Notification', objectName='notification1')
+
         notification = get_notification()
         self._assert_notification(
             notification,
@@ -662,7 +665,8 @@ class EphemeralNotificationsTests(NotificationsBase):
             'Did you watch the air-race in Oshkosh last week?',
             'Phil owned the place like no one before him!',
             'Did really everything in the race work according to regulations?',
-            'Somehow I think to remember Burt Williams did cut corners and was not punished for this.',
+            'Somehow I think to remember Burt Williams did cut corners and '
+            'was not punished for this.',
             'Hopefully the referees will watch the videos of the race.',
             'Burt could get fined with US$ 50000 for that rule-violation :)'
         ]
@@ -677,11 +681,11 @@ class EphemeralNotificationsTests(NotificationsBase):
                 hints=[('x-canonical-append', 'true')]
             )
             notification.show()
-            get_notification = lambda: notify_list.select_single(
+
+            get_notification = lambda: notify_list.wait_select_single(
                 'Notification',
                 objectName='notification1'
             )
-            self.assertThat(get_notification, Eventually(NotEquals(None)))
             notification = get_notification()
             self._assert_notification(
                 notification,
