@@ -20,7 +20,7 @@
 #include "plugin.h"
 #include "AccountsService.h"
 
-#include <QtQml>
+#include <QtQml/qqml.h>
 
 static QObject *service_provider(QQmlEngine *engine, QJSEngine *scriptEngine)
 {
