@@ -18,16 +18,24 @@
 
 #include "../UsersModelPrivate.h"
 
+#include <QDir>
+#include <QSettings>
+#include <QStringList>
+
 namespace QLightDM
 {
 
 UsersModelPrivate::UsersModelPrivate(UsersModel* parent)
   : q_ptr(parent)
 {
-    entries =
+    QSettings settings(QDir::homePath() + "/.unity8-greeter-demo", QSettings::NativeFormat);
+    QStringList users = settings.value("users", QStringList() << "phablet").toStringList();
+
+    Q_FOREACH(const QString &user, users)
     {
-        { "phablet", "Guest", 0, 0, false, false, 0, 0 },
-    };
+        QString name = settings.value(user + "/name", user[0].toUpper() + user.mid(1)).toString();
+        entries.append({user, name, 0, 0, false, false, 0, 0});
+    }
 }
 
 }
