@@ -22,7 +22,9 @@
 // local
 #include "fake_scopes.h"
 #include "fake_categories.h"
-#include "categoryresults.h"
+#include "fake_previewmodel.h"
+#include "fake_previewwidgetmodel.h"
+#include "fake_resultsmodel.h"
 
 // External
 #include <glib-object.h>
@@ -37,8 +39,11 @@ void FakeUnityPlugin::registerTypes(const char *uri)
 
     Q_ASSERT(uri == QLatin1String("Unity"));
 
-    qmlRegisterType<Scopes>(uri, 0, 1, "Scopes");
-    qmlRegisterType<Scope>(uri, 0, 1, "Scope");
-    qmlRegisterType<Categories>(uri, 0, 1, "Categories");
-    qmlRegisterUncreatableType<CategoryResults>(uri, 0, 1, "CategoryResults", "Can't create");
+    qmlRegisterType<Scopes>(uri, 0, 2, "Scopes");
+    qmlRegisterType<Scope>(uri, 0, 2, "Scope");
+    qmlRegisterUncreatableType<Categories>(uri, 0, 2, "Categories", "Can't create Categories object in QML.");
+    qmlRegisterUncreatableType<ResultsModel>(uri, 0, 2, "ResultsModel", "Can't create ResultsModel object in QML.");
+    qmlRegisterType<PreviewModel>(uri, 0, 2, "FakePreviewModel");
+    qmlRegisterUncreatableType<PreviewWidgetModel>(uri, 0, 2, "PreviewWidgetModel", "Can't create new PreviewWidgetModel in QML. Get them from PreviewModel instance.");
+    qmlRegisterUncreatableType<PreviewStack>(uri, 0, 2, "PreviewStack", "Can't create new PreviewStack in QML. Get them from Scope instance.");
 }
